/**
 *
 * @author s071836
 */

package Game;

import Heuristics.ProLog;
import Main.Main;
import Rules.Rules;
import Score.Score;

public class Game {
    private Rules grules = new Rules();
    private Score gscore = new Score();
    private Stock gstock = new Stock();
    private Remaining gremaining = new Remaining();
    private Player[] gplayers = new Player[4];
    private Match gmatch = null;
    private Trick gtrick;
    private boolean foulMove = false;
    private Card.Suit trump;
    private int gameNumber = 0;
    private int trickNumber = 0;                                                    // 0: before first trick. 1-8: first to eighth trick.
    private int gameLeader;
    private int playingTeam; //team which decides to play (0 if player 1/3, 1 if player 2/4)
    
    public static final int NUMOFTRICKS = 8;

    public Game(Match match){
        this.gmatch = match;
        ProLog.setGame(this);
    }

    public Player[] getPlayers(){
        return gplayers;
    }

    /**
     * Main method which is called 16 times from Match and which calls the tricks.
     * @param rules
     * @param score
     * @param players
     * @param stock
     * @param remaining
     */

    public void playGame(Rules rules, Score score, Player[] players, Stock stock, Remaining remaining) {
        Hand hands[] = new Hand[4];

        grules = rules;
        gscore = score;
        gplayers = players;
        gstock = stock;
        gtrick = new Trick(this, gplayers);
        gremaining = remaining;
        gameLeader = gmatch.getGameLeader();
        gameNumber = gmatch.getGameNumber();

        gstock.reset();
        gstock.deal();

        Main.jFrameMain.cardView.updateTableau(gtrick);

        trump = gremaining.getCard().getSuit();

        for (int i=0; i<4; i++){
            hands[i] = players[i].getHand();
            hands[i].sort(trump);
        }

        Main.jFrameMain.cardView.updateCardView(hands);
        Main.jFrameMain.cardView.setCurrentTrump(trump);
        
        // Trick leader must be set before biddingRound() is called
        gtrick.setTrickLeader(gameLeader);

        for (int player=0; player<4; player++) {
            gplayers[player].startGame();
        }
        
        try {
            Thread.sleep(5000);
        } catch (InterruptedException e) {
        }        
        
        if (!biddingRound(trump)) {
            for (int i=0; i<gmatch.getNrOfHumans(); i++) {
                hands[i].sort(trump);
                Main.jFrameMain.cardView.updateCardViewPlayer(i, hands[i]);
            }
        }
       
        for (trickNumber=0; trickNumber<8; trickNumber++) { //Plays the eight tricks of a game
            if (getFoulMove()) {    
                break;
            }

            gtrick.playTrick(trump, gscore, gtrick, grules);
        }

        Main.jFrameMain.scoreView.updateScoreTable(score, gameNumber);
        
        setFoulMove(false);
    }

    /**
     * Asks the players if someone wants to play with the current trump.
     * returns true if someone plays on the given trump
     * return false if everyone passes and someone chose a new trump
     */
    public boolean biddingRound(Card.Suit initialTrump) {

        // Show the turned suit in the GUI as the trump to bid on
        Main.jFrameMain.cardView.setCurrentTrump(initialTrump);

        // Go by every player starting with the game leader (gameLeader ==
        // gtrick.getTrickLeader(), which is used in gtrick.getPlayerId())
        for (int i=0; i<4; i++) {
            Player bidder = gplayers[gtrick.getPlayerId(i)];

            // If a player plays, set the definitive trump suit as the initial
            // trump and set the playing team accordingly
            if (bidder.passOrPlay()) {
                playingTeam = bidder.getPlayerTeam();
                trump = initialTrump;

                return true;
            }
        }

        // Set the gameleaders team as the playing team, and make him choose
        // a trump suit from his hand
        playingTeam = gplayers[gameLeader].getPlayerTeam();
        trump = gplayers[gameLeader].chooseTrump();

        // Show the definitive trump suit in the GUI
        Main.jFrameMain.cardView.setCurrentTrump(trump);

        return false;
    }

    /**
     * Return the number of game (1 to 16).
     * @return Gamenumber.
     */

    public int getGameNumber() {
        return gameNumber;
    }

    public int getPlayingTeam() {
        return playingTeam;
    }

    public int getTrickNumber(){
        return trickNumber;
    }

    public Card.Suit getTrump() {
        return trump;
    }

    public Trick getTrick() { //return method for Virtual
        return gtrick;
    }

    public void setTrumpColour(Card.Suit suit) {
        trump = suit;
    }

    public int getGameLeader() {
        return gameLeader;
    }

    public void setGameLeader(int i) {
        gameLeader = i;
    }

    public void setFoulMove(boolean b) {
        foulMove = b;
    }

    public boolean getFoulMove() {
        return foulMove;
    }
}