/**
 *
 * @author s071836
 */

package Game;

import Rules.Rules;
import Score.Score;
import Main.Main;
import java.awt.CardLayout;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

public class Match implements Runnable {
    private Rules rules = new Rules();
    private Score score = new Score();
    private Stock stock = new Stock();
    private Game game = new Game(this);
    private Remaining remaining = new Remaining();
    private Player[] players = new Player[4];

    private int gameLeader;
    private int gameNumber = 0;
    private int nrOfHumans;

    public static final int NUMOFGAMES = 16;

    public Match(int nrHuman) {
        gameLeader = 0;
        nrOfHumans = nrHuman;
        
        createHands();
    }

    private void createHands () {
       
        // Create nrHuman Human (extends Player) objects for players with GUI
        // input to determine their moves
        for (int i=0; i<nrOfHumans; i++){
            players[i] = new Human(i, stock, game);
        }
        
        // Create 4-nrHuman Virtual (extends Player) objects for players for
        // which the Heuristics class determines their moves
        for (int i=0; i<4-nrOfHumans; i++) {
            players[nrOfHumans+i] = new Virtual(game, score, nrOfHumans+i, 
                    stock);
        }
        
    }

    /**
     * Function which gives the data of the players to stock.
     */
    private void setPlayers() {
        stock.setPlayers(players);
    }
    /**
     *
     */
    private void playMatch() {
        int matchWinner = 1;
        JPanel jPanelContent = (JPanel)Main.jFrameMain.getContentPane();
        
        // Run pre-match initialization for each player
        for (int player=0;player<4;player++) {
            players[player].startMatch();
        }
        
        // Clear the score and game number and reset the remaining pile
        score.resetScore();
        remaining.reset();
        gameNumber = 0;
                
        try {
            Thread.sleep(3000);
        } catch (InterruptedException e) {
        }
        
        // Play Match.NUMOFGAMES games and update game number and game leader
        // after each game
        for (int i=0; i<Match.NUMOFGAMES; i++) {
            game.playGame(rules, score, players, stock, remaining);
            gameNumber++;
            gameLeader = (gameLeader+1)%4;
            game.setGameLeader(gameLeader);
        }
        
        // Set matchWinner to be the team number as displayed in the GUI
        // of the team that won the match, or matchWinner = -1 if it was a tie
        if (score.getTotalScore(0) != score.getTotalScore(1)) {
            matchWinner = (score.getTotalScore(0) > score.getTotalScore(1)) ?
                    1 : 2;
        }
        
        // Display winner in the control panel info text area
        Main.jFrameMain.cardView.setInfoText("The match has ended. " +
                ((matchWinner == -1) ? "Both teams are\ntied!" : "Team " + 
                matchWinner + " is the\nwinner. Congratulations!"));

        // Show the 'End of match' dialog
        JOptionPane.showMessageDialog(null, "Press OK to return to the main "
                + "menu", "End of match", JOptionPane.INFORMATION_MESSAGE);
        
        // Revert the card layout to the main menu card
        ((CardLayout)jPanelContent.getLayout()).previous(jPanelContent);
        
        // Clear the score table
        Main.jFrameMain.scoreView.updateScoreTable(score, -1);
    }

    public int getGameLeader() {
        return gameLeader;
    }

    public int getGameNumber() {
        return gameNumber;
    }

    public int getNrOfHumans() {
        return nrOfHumans;
    }

    @Override
    public void run() {
        setPlayers();
        playMatch();
    }
}
