/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package Game;

/**
 * Represents an (abstract) player
 * @author T.T.M. van Schaijk
 */
public abstract class Player {
    private int playerNumber;
    private int playerTeam;
    private Stock stock;
    protected Hand hand;
    protected Game game;

    /**
     * Should not be called. Player has to be initialized by another constructor.
     */
    protected Player(){
    }

    /**
     * Initializes player
     * @param playerNumber The number the player has. (first player is 1.
     * @param stock A Stock object to be able to deal cards.
     * @param game Game object. Used for hand to get trump.
     */
    protected Player(int playerNumber, Stock stock, Game game){
        this.playerNumber = playerNumber;
        this.playerTeam = playerNumber%2;
        this.stock = stock;
        this.game = game;
        this.hand = new Hand(game);
    }

    /**
     * Determines whether player wants to play or pass.
     * @return True if player wants to play.
     */
    public abstract boolean passOrPlay();

    /**
     * Lets player choose trump.
     * @return Suit of chosen trump.
     */
    public abstract Card.Suit chooseTrump();

    /**
     * Lets player play a card
     * @return Card player wants to play.
     */
    public abstract Card playCard();

    /**
     * Called at start of Match
     */
    public abstract void startMatch();

    /**
     * Called at start of Game
     */
    public abstract void startGame();

    /**
     * Called at start of Trick
     */
    public abstract void startTrick();

    /**
     * Called at end of Trick
     */
    public abstract void endTrick();

    /**
     * Fills the hand of the player with 8 cards.
     */
    public void fillHand() {
        this.hand.empty();

        for(int i=0; i<8; i++) {
            this.hand.addCard(stock.getCard());
        }

        this.hand.sort(game.getTrump());
    }

    /**
     * Asks player if he wants to call honour.
     * @return True if player wants to call honour.
     */
    public abstract boolean promptHonour();

    /**
     * Gets the hand of this player.
     * @return Hand of this player.
     */
    public Hand getHand(){
        return hand;
    }

    /**
     * Returns player number.
     * @return The number of this player.
     */
    public int getPlayerNumber(){
        return playerNumber;
    }

    /**
     * Returns player team.
     * @return The number of the team.
     */
    public int getPlayerTeam(){
        return playerTeam;
    }
}
