/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package Game;
import java.util.*;

/**
 * Holds the cards that will never be in play (used to determine trump cards
 * @author T.T.M. van Schaijk
 */
public class Remaining extends Pile {
    /**
     * Initializes Remaining: Adds cards to the pile and shuffles them.
     */
    public Remaining() {
        // Contains 20 cards
        super(20);

        reset();
    }

    public void reset() {
        super.empty();

        // Add all cards
        for(Card.Suit suit : Card.Suit.values()) {
            for(Card.Rank rank : EnumSet.range(Card.Rank.TWO, Card.Rank.SIX)) {
                super.addCard(new Card(rank, suit));
            }
        }

        // Shuffle cards
        this.shuffle();
    }

    /**
     * Turn the top card.
     * @return
     */
    public Card turn() {
        return super.getCard();
    }

    /**
     * Shuffles the cards, putting the first clubs on top.
     */
    @Override public void shuffle() {
        ArrayList<Card> tmpCards = new ArrayList<Card>();
        Card topCard = null;
        int cardCount = super.getCardCount();

        super.shuffle();

        for(int i=0; i<cardCount; i++) {
            Card tmpCard = super.getCard();
            if (tmpCard.getSuit() == Card.Suit.CLUBS &&
                tmpCard.getRank() == Card.Rank.TWO && topCard == null) {
                topCard = tmpCard;
            } else {
                tmpCards.add(tmpCard);
            }
        }
        
        super.empty();
        super.addCard(topCard);
        
        for (int i=0; i<cardCount-1; i++) {
            super.addCard(tmpCards.remove(0));
        }
    }
}