/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package Game;

import Game.Card.Suit;
import Rules.Rules;
import Score.Score;
import Main.Main;

/**
 *
 * Represents a trick in the game.
 * @author T.T.M. van Schaijk
 */
public class Trick {
    private Pile cards = new Pile(4);
    private Game game;
    private Player[] players = new Player[4];
    private int trickLeader;
    public Suit trumpSuit;
    private int activePlayer = 0;

    public Trick(Game game, Player[] gplayers) {
        this.game = game;
        players = gplayers;
        trickLeader = game.getGameLeader(); //Sets trickleader to gameleader at the first trick of a game
    }

    /**
     *
     * @param trump
     * @param score
     * @param gtrick
     * @param rules

     */
    public void playTrick(Card.Suit trump, Score score, Trick gtrick, Rules rules) {
        Player currentPlayer;
        Hand playerHands[] = new Hand[4];
        int foul;

        trumpSuit = trump;

        // This loop plays the actual trick player by player
        // Trick.getPlayerId(i) gets the player id by the position i relative
        // to the trick owner

        for (int i=0; i<4; i++) {
            activePlayer = getPlayerId(i);

            // Save a reference to the Player object representing the currently
            // playing player, along with a deep copy of his/her Hand before
            // playing a card.
            currentPlayer = players[activePlayer];            
            playerHands[activePlayer] = new Hand(game);

            for (int j=0; j<players[activePlayer].getHand().getCardCount(); j++) {
                playerHands[activePlayer].addCard(players[activePlayer].getHand().getCard(j));
            }

            // Prepare the player for the trick
            currentPlayer.startTrick();

            // Make the player pick a card and add it to the trick cards
            cards.addCard(currentPlayer.playCard());

            // Update the players hand and the tableau in the GUI to reflect the changes
            Main.jFrameMain.cardView.updateCardViewPlayer(activePlayer, players[activePlayer].getHand());
            Main.jFrameMain.cardView.updateTableau(this);

            // Finally sleep for a while to give the player a chance to look
            // at the changes in the GUI
            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {

            }

        }

        foul = rules.checkRules(gtrick, trump, playerHands);

        // If there were no fouls
        if (foul == Rules.NOFOUL) {

            // We either calculate the score of this trick and output it
            score.checkScore(players[trickLeader], game, trump);

            // Set the new trick leader according to this tricks winner
            // determined by score
            trickLeader = score.getWinner();

            // And give every player the chance to do stuff at the end
            // of the trick (true action depends on Human or Virtual)
            for (int i=0; i<4; i++) {
                players[getPlayerId(i)].endTrick();
            }

        } else { // If there were fouls
            
            // Broadcast a message
            Main.jFrameMain.cardView.setInfoText("Team " +
                    (players[getPlayerId(foul)].getPlayerTeam()+1) + " has " +
                    "comitted a foul and\nforfeits all " + Score.TOTALSCORE +
                    " points and honours\nto the opposing team.");
            
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
            }

            // Set the foul move flag in Game
            game.setFoulMove(true);

            // Initiate appropriate consequences for the score of the player
            // who committed the foul (and his/her teammate)
            score.foulMove(players[getPlayerId(foul)],gtrick.game);
            
        }

        //pile leegmaken anders kan geen nieuwe kaart worden geaccepteerd (in de volgende trick)
        cards.empty();
        
        try {
            Thread.sleep(1000);
        } catch(InterruptedException e) {

        }

        Main.jFrameMain.cardView.updateTableau(this);
    }

    /**
     * Get the position of who has the lead of the untilpos first players who
     * played this trick.
     * @param untilpos
     * @return The playernumber of the trickowner. -1 if error.
     */
    public int getTrickOwner(int untilpos) {
        int owner = 0;

        for (int i=1; i<untilpos; i++) {
            if (!cards.getCard(owner).higher(cards.getCard(i), trumpSuit)) {
                owner = i;
            }
        }

        return owner;
    }

    /**
     * Get the position of the winner of the trick.
     */

    public int getTrickOwner() {
        return getTrickOwner(4);
    }

    /**
     * Get the card the trick leader has played this trick
     * @return
     */

    public Card getTrickLeaderCard() {
        return cards.getCard(0);
    }

    /**
     * Function to return the highest trump card within the
     * cards played by the first untilpos players in this trick.
     * @return highest trump card or null if no trump cards were played
     */
    public Card getHighestTrumpCard(int untilpos) {
        Card highestCard = null;

        for(int player=0;player<untilpos;player++) {
            if(this.getCard(player).getSuit() == trumpSuit) {
                if(this.getCard(player).higher(highestCard, trumpSuit)) {
                    highestCard = this.getCard(player);
                }
            }
        }

        return highestCard;
    }

    /**
     * Determines player id from the players relative position to the trick leader
     * and whoever is the trick leader.
     * @param playerNumber Number of player (0-3)
     * @return Position of requested player (0-3)
     */
    public int getPlayerId(int position) {
        return (getTrickLeader() + position) % 4;
    }

    /**
     * Determines the relative position of player with playerId to the trick
     * leader.
     * @param playerId the player id of the player (0-3)
     * @return the relative position of the player (0-3)
     */
    public int getTrickPosition(int playerId) {
        return (playerId - getTrickLeader() + 4)%4;   // Add 4 before the mod 4 to avoid a negative result
    }

    /**
     * @return the player id of the player whose turn it currently is. Only makes
     * sense when called in playTrick() for loop.
     */
    public int getActivePlayerId(){
        return activePlayer;
    }

    /**
     * Adds a card to the trick.
     * @param card Card to add.
     */
    public void addCard(Card card) {
        cards.addCard(card);
    }
    
    /**
     * Gets requested card from trick.
     * @param cardNumber number of requested card.
     * @return requested card.
     */
    public Card getCard(int cardNumber) {
        return cards.getCard(cardNumber);
    }

    /**
     * Gets the player id of the current trick leader.
     * @return position of trick leader (0-3)
     */
    public int getTrickLeader() {
        return trickLeader;
    }

    /**
     * Sets the current trick leader.
     * @param trickLeader The new trickleader.
     */

    public void setTrickLeader(int trickLeader) {
        this.trickLeader = trickLeader;
    }

    public int getPosition(int playerNumber){
        return (playerNumber + 4 - trickLeader)%4;
    }
}