/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package Game;
import Heuristics.*;
import Score.*;

/**
 * Implements non-human players.
 * @author T.T.M. van Schaijk
 */
public  class Virtual extends Player {
    Heuristics heuristics = new Heuristics(this);
    Score score;

    /**
     * Normal constructor
     * @param game The game object, used for getting trump colours.
     * @param score The Score object. Used for determining whether Virtual wants to call honour.
     * @param playerNumber The number of the player. Used by heuristics.
     * @param stock The stock object, used fill the players hand.
     */
    protected Virtual(Game game, Score score, int playerNumber, Stock stock) {
        super(playerNumber, stock, game);
        this.game = game;
        this.score = score;
        ProLog.initialise();
    }

    /**
     * Determines wheter computer wants to play or pass.
     * @return True if computer wants to play.
     */
    @Override
    public boolean passOrPlay() {
        return heuristics.playOrPass(0, super.getHand(), game.getTrump());
    }

    /**
     * Determines which trumpcolour computer prefers
     * @return Suit of the preferred trump
     */
    @Override
    public Card.Suit chooseTrump() {
        return heuristics.chooseTrump(super.getHand(), game.getTrump());
    }

    /**
     * Determines which card computer wants to play
     * @return Card computer wants to play.
     */
    @Override
    public Card playCard() {
        Card card = heuristics.playCard(game.getTrick(), game.getTrump(),
                super.getHand(), 
                game.getTrick().getTrickPosition(super.getPlayerNumber()));
        super.hand.removeCard(card);
        return card;
    }

    /**
     * Determines whether computer wants to call honour.
     * @return True if computer wants to call honour.
     */
    @Override
    public boolean promptHonour() {
        return heuristics.callHonour(0, super.getHand(), game.getTrump(),
               score.getGameScore(super.getPlayerTeam(), game.getGameNumber()),
               score.getCurrentHonour(), score.getPossibleHonour());
    }

    /**
     * Called at start of Match
     */
    @Override
    public void startMatch() {
        heuristics.startMatch();
    }

    /**
     * Called at start of Game
     */
    @Override
    public void startGame() {
        heuristics.startGame(game.getTrump());
    }

    /**
     * Called at start of Trick
     */
    @Override
    public void startTrick() {
        heuristics.startTrick(game.getTrick());
    }

    /**
     * Called at end of Trick
     */
    @Override
    public void endTrick() {
        heuristics.endTrick(game.getTrick());
    }
}
