/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package Heuristics;
import Game.*;
import Rules.Rules;
import java.util.Hashtable;
import jpl.*;

/**
 *
 * @author T.T.M. van Schaijk
 */

/**
 * Some functions to make code more readable
 * @author T.T.M. van Schaijk
 */
public class ProLog {//TODO check which statements can be retracted when.

    /*
     * Definition of ProLog terms:
     * currentHand/1    pointer to hand object      1: pointer to hand
     * hand/3           Contents of players hand    1: playerNumber     2: suit of card     3: rank of card
     * higher/4         Overall order of cards      1: suit of highest  2: rank of highest  3: suit of lowest   4: rank of lowest
     * higherNormal/2   Normal order of cards       1: highest cardrank 2: lowest cardrank
     * higherNoTrump/2  Notrump order of cards      1: highest cardrank 2: lowest cardrank
     * higherTrump/2    Trump order of cards        1: highest cardrank 2: lowest cardrank
     * leaderCard/3     First card in the trick     1: playerNumber     2: suit of the card 3: rank of the card
     * ownerCard/3      Highest card in the trick   1: player that played card  2: suit of card     3: rank of card
     * partner/2        Partners                    1: partner 1        2: partner 2
     * thisTrick/3      Cards in the current trick  1: playerNumber     2: suit of card     3: rank of card
     * trick/4          Cards in all tricks         1: trickNumber      2: playerNumber     3: suit of card     4: rank of card
     * trump/1          Suit of trump               1: suit of trump
     */

    private static int trickNumber = 0;
    private static Card.Suit trump = Card.Suit.CLUBS;
    private static boolean started = false;
    private static Game game = null;

    /**
     * Should not be called as this is a singleton.
     */
    public ProLog(){
    }

    /**
     * Asserts the everlasting rules.
     */
    public static void initialise(){
        if(started)
            return;

         /*
         * If both don't have leaderSuit, they will certainly not win the round
         * and the return value can be false. I have not included this knowledge
         * as this would clog the knowledgebase.
         */

        // Set test for higher cards
        ProLog.assertt("higher(A,B,C,D) :- ("
                + "assert(cardA(A,B)),"
                + "assert(cardB(C,D)),"
                + "jpl_call('Heuristics.ProLog', higher, [], _),"
                + "higherCard(C,D),"
                + "retract(higherCard(_,_)),"
                + "retract(cardB(_,_)),"
                + "retract(cardA(_,_))"
                + ")");

        // Set test for playable cards
        ProLog.assertt("playable(A,B,C) :- ("
                + "jpl_call('Heuristics.ProLog', playable, [], _),"
                + "playableCard(A,B,C),"
                + "retract(playableCard(_,_,_))"
                + ")");

        // Set partners
        ProLog.assertt("partner(0,2)");
        ProLog.assertt("partner(2,0)");
        ProLog.assertt("partner(1,3)");
        ProLog.assertt("partner(3,1)");

        // Load user file.
        ProLog.query("consult('player" + ".pl')");

        // Prevent multiple executions
        started = true;
    }

    /**
     * Called at start of game.
     * @param trump Trump for that game.
     */
    public static void startGame(Card.Suit trump){
        ProLog.trump = trump;

        // Set trump in ProLog
        ProLog.retract("trump", 1);
        ProLog.assertt("trump(" + trump.ordinal() + ")");
    }

    /**
     * Called at start of trick.
     * @param trick The current trick as know to Virtual.
     */
    public static void startTrick(Trick trick, int player){
        ProLog.retract("thisTrick", 3);                                  // Remove all previous knowledge of this trick from the database.
        for(int i=0; i<4; i++){
            if(trick.getCard(i) != null){
                ProLog.assertt("thisTrick(" + player + "," +
                        trick.getCard(i).getSuit().ordinal() +
                        "," +
                        trick.getCard(i).getRank().ordinal() +
                        ")");
            }
        }

        ProLog.retract("leaderCard", 3);
        if(trick.getTrickLeaderCard() != null){
            ProLog.assertt("leaderCard(" + player + "," +
                        trick.getTrickLeaderCard().getSuit().ordinal() +
                        "," +
                        trick.getTrickLeaderCard().getRank().ordinal() +
                        ")");
        }
    }

    /**
     * Passes trick information to ProLog
     * @param trick A full trick
     */
    public static void endTrick(Trick trick){
        trickNumber++;
        String query = "trick(" + trickNumber + ", ";
        for(int i=0; i<4; i++){
            ProLog.assertt("trick(" + trickNumber + ", " + i + "," +
                    trick.getCard(i).getSuit().ordinal() + "," +
                    trick.getCard(i).getSuit().ordinal() + ")");
        }
    }

    /**
     * Executes a query to ProLog
     * @param query The query to send to ProLog.
     */
    public static void query(String query){
        new Query(query + ".").oneSolution();
    }

    /**
     * Determines wheter query has a solution.
     * @param query query to execute.
     * @return True if query has solution.
     */
    public static boolean hasSolution(String query){
        try{
            return (new Query(query + ".")).hasSolution();
        } catch (Exception e) {
            return false;
        }
    }

    /**
     * Returns one solution to the provided query-string
     * @param query String containing the query to execute (without ".")
     * @return A hashtable containing one solution to the problem. Individual variables can be retreived from this table.
     */
    public static Hashtable oneSolution(String query){
        try{
            return (new Query(query + ".")).oneSolution();
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * Gets a variable from given Hashtable.
     * @param s Hashtable containing all variables that ProLog returns
     * @param variable String defining requested variable
     * @return Value of variable
     */
    public static Object oneSolution(Hashtable s, String variable){
        try{
            return s.get(variable);
        } catch (Exception e){
            System.err.println("WARNING: No solution found.");
            return null;
        }
    }

    /**
     * Convert an Integer object to an int (JPL returns an jpl.Integer
     * @param s Hashtable containing all variables that ProLog returns
     * @param variable String defining requested variable
     * @return Integer value of ProLog variable
     */
    public static int oneSolutionInt(Hashtable s, String variable){
        Object val = oneSolution(s, variable);
        if(val == null){
            return -1;
        }
        return java.lang.Integer.parseInt(val.toString());
    }

    /**
     * Converts an int to a Card.Suit
     * @param s Hashtable containing all variables that ProLog returns
     * @param variable String defining requested variable
     * @return Suit of the card represented by variable
     */
    public static Card.Suit oneSolutionSuit(Hashtable s, String variable){
        int solution = oneSolutionInt(s, variable);
        if(solution == -1)
            return null;
        return Card.Suit.values()[solution];
    }

    /**
     *
     * @param s Hashtable containing all variables that ProLog returns
     * @param variable String defining requested variable
     * @return Rank of card represented by variable
     */
    public static Card.Rank oneSolutionRank(Hashtable s, String variable){
        int solution = oneSolutionInt(s, variable);
        if(solution == -1)
            return null;
        return Card.Rank.values()[solution];
    }

    /**
     * Returns a card that can be played.
     * @param player The number of the player that has to play
     * @return A card that player wants to play
     */
    private static Card queryCard(int player){
        // Create ProLog variables
        Variable SuitToPlay = new Variable("SuitToPlay");
        Variable RankToPlay = new Variable("RankToPlay");

        // Build a query object
        Query q = new Query (
                "play",
                new Term[]{
                    new jpl.Integer(player),
                    SuitToPlay,
                    RankToPlay
                }
        );

        // Execute query and create a card that is a solution
        Hashtable s = q.oneSolution();
        Card.Suit suit = oneSolutionSuit(s,"SuitToPlay");
        Card.Rank rank = oneSolutionRank(s,"RankToPlay");
        Card card = null;
        if(suit != null && rank != null){
            card = new Card(rank, suit);
        }

        return card;
    }

    /**
     * Asserts the compound in the string. (As assert is a keyword in java, assertt is used)
     * @param string The compound to assert.
     */
    public static void assertt(String string){
        try{
            ProLog.query("assert(" + string + ")");
        } catch (Exception e) {
            System.err.println("Error in ProLog.assertt(" + string + ")");
        }
    }

    /**
     * Retracts the given statement.
     * @param string Statement to retract
     * @param nrArg Number of arguments statement has.
     */
    public static void retract(String string, int nrArg){
        String arg = "";
        if(nrArg > 0){
            arg += "_";
        }
        for(int i=1; i<nrArg; i++){
            arg += ",_";
        }
        try{
            Query q = new Query(string + "(" + arg + ")");
            while(q.hasSolution()){
                ProLog.query("retract(" + string + "(" + arg + "))");
                q.rewind();
            }
        } catch (Exception e) {
            System.err.println("Warning in ProLog.retract(" + string + "," + nrArg + "): " + string + " does not exist");
        }
    }

    /**
     * Called to let player play a card
     * @param position Position of the player
     * @param player Number of the player
     * @param hand Hand of the player
     * @param trick Current trick
     * @return The card that player wants to play
     */
    public static Card playCard(int position, int player, Hand hand, Trick trick){
        // Known: trump, trick

        // Set hand
        ProLog.retract("hand", 3);
        for(int i=0; hand.getCard(i)!=null; i++){                               // Fill hand
            ProLog.assertt("hand(" + player + "," +
                    hand.getCard(i).getSuit().ordinal() + "," +
                    hand.getCard(i).getRank().ordinal() + ")");
        }
        startTrick(trick, player);

        System.err.println("ProLog: " + oneSolution(""
                + "hand(1, SuitToPlay, RankToPlay), "
                + "leaderCard(_,LeaderSuit,_),"
                + "not(setof([LeaderSuit], trump(LeaderSuit), L)), length(L,N), (N=0),"
                + "SuitToPlay = LeaderSuit"
                //+ "playFollowSuit(SuitToPlay, RankToPlay,LeaderSuit)"
                + ""));
        Card cardPlayed = queryCard(player); //TODO Query play.

        // Check for errors
        if(cardPlayed == null){
            System.err.println("ERROR: No card played!");
            return null;
        }

        // Set OwnerCard
        if(position == 0){
            ProLog.retract("ownerCard", 3);
            ProLog.assertt("ownerCard(" + player + "," +
                    cardPlayed.getSuit().ordinal() + "," +
                    cardPlayed.getRank().ordinal() + ")");
            ProLog.retract("leaderCard", 3);
            ProLog.assertt("leaderCard(" + player + "," +
                    cardPlayed.getSuit().ordinal() + "," +
                    cardPlayed.getRank().ordinal() + ")");
        } else {
            ProLog.retract("ownerCard", 3);
            for(int i=0; i<4; i++){                                             // Determine ownerCard
                if(!ProLog.hasSolution("higher(A,B," +
                        cardPlayed.getSuit().ordinal() + "," +
                        cardPlayed.getRank().ordinal() +
                        "), thisTrick(_,A,B)")){
                    // No higher card then this card
                    ProLog.assertt("ownerCard(" + player + "," +
                        cardPlayed.getSuit().ordinal() + "," +
                        cardPlayed.getRank().ordinal() + ")");
                }
            }
        }

        return cardPlayed;
    }

    /**
     * Helper function for the ProLog function higher defined below. This function determines all cards higher than the provided card.
     * higher(A,B,C,D) :- (
     *      assert(cardA(A,B)),
     *      assert(cardB(C,D)),
     *      jpl_call('Heuristics.ProLog', higher, [], _),
     *      higherCard(C,D),
     *      retract(higherCard(_,_)),
     *      retract(cardB(_,_)),
     *      retract(cardA(_,_)).
     */

    public static void higher(){
        // Create ProLog variables
        Variable SuitA = new Variable("SuitA");
        Variable RankA = new Variable("RankA");
        Variable SuitB = new Variable("SuitB");
        Variable RankB = new Variable("RankB");

        // Build query objects
        Query qA = new Query (
                "cardA",
                new Term[]{
                    SuitA,
                    RankA
                }
        );
        Query qB = new Query (
                "cardB",
                new Term[]{
                    SuitB,
                    RankB
                }
        );

        // Make cards
        Hashtable sol = qA.oneSolution();
        Card.Suit suitA = oneSolutionSuit(sol,"SuitA");
        Card.Rank rankA = oneSolutionRank(sol,"RankA");
        Card cardA = null;
        Card cardB = null;
        if(suitA != null && rankA != null){
            cardA = new Card(rankA, suitA);
        }
        System.err.println("A cardA: " + cardA);

        Hashtable[] solutions;
        try{
            solutions = qB.allSolutions();
        } catch (Exception e){
            System.err.println("WARNING: No solution found.");
            return;
        }

        Card.Suit suitB;
        Card.Rank rankB;
        for(int s=0; s<solutions.length; s++){
            suitB = Card.Suit.values()[java.lang.Integer.parseInt(
                    solutions[s].get("SuitB").toString())];
            rankB = Card.Rank.values()[java.lang.Integer.parseInt(
                    solutions[s].get("RankB").toString())];
            if(suitB != null && rankB != null){
                cardB = new Card(rankB, suitB);
            }

            // Return true if A>B
            if(cardA != null && cardB != null){
                if(!cardA.higher(cardB, trump)){
                    System.err.println("A CardB: " + cardB);
                    ProLog.assertt("higherCard(" +
                            cardB.getSuit().ordinal() + "," +
                            cardB.getRank().ordinal() + ")");
                }
            }
        }
    }

    /**
     * Function to get a reference to the game object that is referenced often in order to get the current player etc.
     * @param game pointer to the current game object
     */
    public static void setGame(Game game){
        ProLog.game = game;
    }

    /**
     * Helperfunction for the prolog function playable(). This selects all cards that are playable by the current player.
     * playable(A,B,C) :- (
     *      jpl_call('Heuristics.ProLog', playable, [], _),
     *      playableCard(A,B,C),
     *      retract(playableCard(_,_,_)).
     */

    public static void playable(){

        Trick currentTrick = game.getTrick();
        int playerId = currentTrick.getActivePlayerId();
        
        Pile playableCards = (new Rules()).getPlayableCards(currentTrick, trump, game.getPlayers()[playerId].getHand(), currentTrick.getPosition(playerId));

        for(int i=0; i<playableCards.size(); i++){
            ProLog.assertt("playableCard("
                    + game.getTrick().getActivePlayerId() + "," +
                    + ((Card) playableCards.get(i)).getSuit().ordinal() + ","
                    + ((Card) playableCards.get(i)).getRank().ordinal() + ")"
                    );
        }
    }
}

