/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package Rules;

import Game.Card;
import Game.Hand;
import Game.Trick;
import Game.Pile;

public class Rules {

    public static final int NOFOUL = 4;

    /**
     * Go by each players card (starting with the trick leader) which he/she
     * played in the trick and check if the card is playable.
     * @param trick         trick
     * @param trumpCard     card that determines the trump
     * @param playerHands[] array of Hands for the players by id (0..3)
     * @return              integer of relative position of the player to the trick leader who committed a foul, or NOFOUL if no foul was committed.
     */
    public int checkRules(Trick trick, Card.Suit trumpCard, Hand playerHands[]) {
        for (int i=0; i<4; i++) {
            if (!getPlayableCards(trick, trumpCard, playerHands[trick.getPlayerId(i)], i).contains(trick.getCard(i))) {
                return i;
            }
        }

        return NOFOUL;
    }

    public Pile getPlayableCards(Trick trick, Card.Suit trump, Hand playerHand, int position) {
        Pile playableCards = new Pile(8);
        Card trumpCard = trick.getHighestTrumpCard(position);                   // find the highest trump card played in the trick by the players before you

        if (position == 0) {                                                    // if you are the trick leader, you can play anything from your hand
            return playerHand.getAllCards();
        }

        if (trick.getTrickLeaderCard().getSuit() == trump) {                    //if the leading suit is trump
            if (playerHand.hasSuit(trump)) {                                    //if you have trump card(s)
                playableCards = playerHand.getHigherTrumpCard(trumpCard);       //get all the cards that are higher in rank than the highest played trump

                return  playableCards.isEmpty() ?
                        playerHand.getCardBySuit(trump) :                       // if you dont have higher trump cards, play any other trump card
                        playableCards;                                          // if you do have higher trump cards, play those
            } else {                                                            //leading suit is trump, but you do not have trump
                return playerHand.getAllCards();                                // play any card from your hand
            }
        }

        if (playerHand.hasSuit(trick.getTrickLeaderCard().getSuit())) {         //if you can follow suit
            return playerHand.getCardBySuit(trick.getTrickLeaderCard().getSuit());  // do so
        }

        if (trumpCard != null) {                                                // if anyone trumped this non-trump leading suit trick
            playableCards = playerHand.getHigherTrumpCard(trumpCard);

            if (playableCards.isEmpty()) {                                      // if you dont have any higher trump cards
                playableCards = playerHand.getCardBySuitExcept(trump);

                return  playableCards.isEmpty() ?
                        playerHand.getAllCards() :                              // and if you dont have any other cards than trump cards, play any card
                        playableCards;                                          // and if you do have other cards, play any of those non-trump cards
            } else {
                return playableCards;                                           // if you do have higher trump cards, play those
            }
        }

        playableCards = playerHand.getCardBySuit(trump);

        if (playableCards.isEmpty()) {                                          // if you dont have any trumps to play, you can play any card
            return playerHand.getAllCards();
        }

        return  (trick.getTrickOwner(position) == (position-2)) ?               // if youre partner is in the lead
                playerHand.getAllCards() :                                      // you can play anything you want
                playableCards;                                                  // else you must play a trump
    }
}
